function classifyResult = challenge(recordName)
%
% Sample entry for the 2017 PhysioNet/CinC Challenge.
%
% INPUTS:
% recordName: string specifying the record name to process
%
% OUTPUTS:
% classifyResult: integer value where
%                     N = normal rhythm
%                     A = AF
%                     O = other rhythm
%                     ~ = noisy recording (poor signal quality)
%
% To run your entry on the entire training set in a format that is
% compatible with PhysioNet's scoring enviroment, run the script
% generateValidationSet.m
%
% The challenge function requires that you have downloaded the challenge
% data 'training_set' in a subdirectory of the current directory.
%    http://physionet.org/physiobank/database/challenge/2017/
%
% This dataset is used by the generateValidationSet.m script to create
% the annotations on your training set that will be used to verify that
% your entry works properly in the PhysioNet testing environment.
%
%
% Version 1.0
%
%
% Written by: Chengyu Liu and Qiao Li January 20 2017
%             chengyu.liu@emory.edu  qiao.li@emory.edu
%
% Last modified by:
%
%


%% AF determination (original sample code)
%{
[tm,ecg,fs,siginfo]=rdmat(recordName);
[QRS,sign,en_thres] = qrs_detect2(ecg',0.25,0.6,fs);
if length(QRS)<6
else
    RR=diff(QRS')/fs;
    AFEv = comput_AFEv(RR);
    if AFEv>1
        classifyResult = 'A';
    end
end
%}
%% Model 1 AFEv+sampent+RR_sd -> Neural Net
%{
    load('.\Model\Model1 AFEv_samp_sd_NN') % will load a neural net
    [~,ecg,fs,~]=rdmat(recordName);
    mask = UNSW_ArtifactMask(ecg,[-5,5],0,fs);
    [~,RR] = UNSW_QRSDetector(ecg,fs,mask,0);
    RR = RR/fs;
    AFEv = comput_AFEv(RR');
    sampent = cjy_feature(RR);
    RR_sd = std(RR);
    
    feature = [AFEv,sampent,RR_sd];
    [~,temp_result] = max(net(feature'));
    
    % convert
    switch temp_result
        case 1
            classifyResult = 'N';
        case 2
            classifyResult = 'A';
        case 3
            classifyResult = 'O';
        case 4
            classifyResult = '~';
    end
 %}
%% Model 3 model1+norm_mse
%{
load('Model10 ensembleNN+30feature')
    [~,ecg,fs,~]=rdmat(recordName);
    mask = UNSW_ArtifactMask(ecg,[-5,5],0,fs);
    [qrs,RR] = UNSW_QRSDetector(ecg,fs,mask,0,0);
    feature = cjy_feature(ecg,qrs,RR,fs);
    temp_result = zeros(1,numel(net));
    for i = 1:numel(net)
        [~,temp_result(i)] = max(net{i}(feature'));
    end
    temp_result = mode(temp_result);
    % convert
    switch temp_result
        case 1
            classifyResult = 'N';
        case 2
            classifyResult = 'A';
        case 3
            classifyResult = 'O';
        case 4
            classifyResult = '~';
        otherwise
            classifyResult = '~';
    end
%}
%% Model 11 Original+DiffUnits+MajVoting
load('Model11 original_result_hidunits')
    [~,ecg,fs,~]=rdmat(recordName);
    net = net(2,:);
    mask = UNSW_ArtifactMask(ecg,[-5,5],0,fs);
    [qrs,RR] = UNSW_QRSDetector(ecg,fs,mask,0,0);
    feature = cjy_feature(ecg,qrs,RR,fs);
    temp_result = zeros(1,numel(net));
    for i = 1:numel(net)
        [~,temp_result(i)] = max(net{i}(feature'));
    end
    temp_result = mode(temp_result);
    % convert
    switch temp_result
        case 1
            classifyResult = 'A';
        case 2
            classifyResult = 'N';
        case 3
            classifyResult = 'O';
        case 4
            classifyResult = '~';
        otherwise
            classifyResult = '~';
    end

