function [feature,wave_group] = cjy_feature(signal,qrs,RR,fs)
 % 3/18 model 2 contain 8 features
 % 3/19 contain another 6 RWE
 
 RR = RR/fs;
 feature = zeros(1,4);
 % RR feature
 feature(1) = comput_AFEv(RR');
 feature(2) = sampent(RR);
 feature(3) = std(RR);
 feature(4) = mean(RR);
 
 % waveform feature
 [wave_mat,wave_temp,wave_group] = find_wave(signal,qrs,fs);

 norm_mse = cjy_mse(wave_mat,wave_temp,wave_group); % 5 feature
%  RWE = wavelet_entropy(wave_temp); % 6 feature
%  feature7 = Stephen_mse(wave_mat,wave_temp,fs); % 7 feature
 hf_mean = hf(signal,fs); % 1 feature
 poincare_std = Poincare(RR); % 2 feature
 [duration,peak] = PQRST(wave_temp,fs);% 5 feature
 feature = [feature norm_mse hf_mean poincare_std duration peak];
 
 
 
function stdRR = Poincare(RR)
    dRR = diff(RR);
    RR = [RR(1:end-1);RR(2:end)];
    dRR = [dRR(1:end-1);dRR(2:end)];
    std1 = sqrt(mean(sum((bsxfun(@minus,RR,mean(RR))).^2,2)));
    std2 = sqrt(mean(sum((bsxfun(@minus,dRR,mean(dRR))).^2,2)));
    stdRR = [std1 std2];
    
function sampent = sampent(RR,varargin)
    % Input
    % RR    input series
    % m     the embedding dimension of space
    % th    the threshold for determine if it is similar

    % Output
    % sub_feature 
   coe = 1;
 switch nargin
     case 1
            m = 2;
            th = coe*std(RR);
     case 2
            m  = varargin{1};
            th = coe*std(RR);
     case 3
            m  = varargin{1};
            th = varargin{2};
     otherwise
         error('wrong input\n');
 end

    l = numel(RR);
    S = zeros(m+1,l-m+1);
    A = 0; % number of repeated pairs with the length of m+1
    B = 0; % number of repeated pairs with the length of m

    % Case 1: length of m
    for i = 1:m
        S(i,:) = RR(i:end+i-m);
    end
        S(m+1,1:end-1) = RR(m+1:end);

    for i = 1:l-m
        D = abs(bsxfun(@minus,S(:,i),S(:,i+1:end)));
        % we use Cheb Distance
        D1 = max(D(1:m,:));
        D2 = max([D1(1:end-1);D(m+1,1:end-1)]);
        B = B + sum(D1<th);
        A = A + sum(D2<th);
    end

    sampent = (A/B); 
    
function [wave_mat,wave_temp,k] = find_wave(signal,qrs,fs)
    % Extract the wave segment and find the wave template 
    % using k-means
    
    % ouput
    % wave      - M*N double, All wave segment, M the number of segment
    %             N the length of segment, defaut 1s.              
    % wave_temp - 1*N double, The template waveform synthesised by k-means.
    % k         - 1*M int,The group index of each wave segment.
    left = 0.5*fs;
    right = 0.5*fs;
    QRS = 0.45*fs:0.55*fs; % QRS window

    m_id = find(qrs>left & qrs < length(signal) - right);
    m = numel(m_id);
    wave_mat =  zeros(m,left+right+1);
    for j = 1:m
       wave_mat(j,:) = signal(qrs(m_id(j))-left:qrs(m_id(j))+left);
    end

    % using k-means to find the template
    rng(1) % for reproductivity
    [k,C] = kmeans(wave_mat,2);
    k_max = mode(k);
    wave_temp = C(k_max,:);

function stat_mse = cjy_mse(wave_mat,wave_temp,k)
    % Output
    % 
   % the following four indexes are for normalization
   amp_temp = mean(abs(wave_temp));
   mse = zeros(1,numel(k));
   
   % compute the normalized mse
   for j = 1:numel(k)
       mse(j) = mean(abs(wave_temp-wave_mat(j,:)))/sqrt((amp_temp*mean(abs(wave_mat(j,:)))+eps));      
   end
   
   stat_mse(1) = min(mse);
   stat_mse(2) = max(mse);
   stat_mse(3) = mean(mse);
   stat_mse(4) = mode(mse);
   k_max = mode(k);
   stat_mse(5) = sum(k == k_max)/numel(k); 
   
   function  RWE = wavelet_entropy(wave_temp)
   % wavelet decomposition, wavedec
    [C,L] = wavedec(wave_temp,5,'sym4');
    sumL = cumsum(L);
    R = zeros(1,6);
    R(1) = sum(C(1:sumL(1)).^2);
    for j = 1:5
        R(j+1) = sum(C(sumL(j):sumL(j+1)).^2);
    end
    RWE = R/sum(R);
    
   function    feature7 = Stephen_mse(wave_mat,wave_temp,fs)
    feature7 = zeros(1,7);
    M = size(wave_temp,1); 
    feature_mat = zeros(M,5);
    QRS = round(0.45*fs:0.55*fs);
    norm_wave = norm(wave_temp);
    peak_height = abs(median(wave_temp(QRS)));
    
    % adding the new 7 features
    % 7 features are proposed by Professor Stephen in
    % his paper in signal quality in 2012
    % Redmond, S.J., et al., Electrocardiogram signal quality measures 
    % for unsupervised telehealth environments. 
    % Physiological Measurement, 2012. 33(9): p. 1517.

    for j = 1:M
        err = norm(wave_temp-wave_mat(j,:));
        feature_mat(j,1) = err/norm_wave;
        feature_mat(j,2) = err/peak_height;
        feature_mat(j,3) = norm(wave_temp(QRS)-wave_mat(j,QRS))/norm_wave;
        cortemp1 = corrcoef(wave_temp,wave_mat(j,:));
        feature_mat(j,4) = cortemp1(1,2);
        cortemp2 = corrcoef(wave_temp(QRS),wave_mat(j,QRS));
        feature_mat(j,5) = cortemp2(1,2);
    end
    feature7(1:5) = mean(feature_mat);
    feature7(6) = median(feature_mat(:,4));
    feature7(7) = median(feature_mat(:,5));
    
function hfmean = hf(signal,fs)     
    [b,a] = ellip(5,0.5,80,40/(fs/2),'high');
    hfcleandata = myfiltfilt(b,a,signal);
    % smooth the power
    [b]=hamming(round(0.05*fs));b=b/sum(abs(b));[a]=1;
    b=b./sum(abs(b));
    hfpower=real(myfiltfilt(b,a,hfcleandata.^2).^0.5); 
    hfmean = mean(hfpower);
    
function [duration,peakheight] = PQRST(wave_temp,fs)
    % extract wavetime and waveamp
    % duration 1*3 double, the duration time of P, QRS and T wave
    % peakheight 1*3 double, the height of P, QRS and T wave
    
    duration = zeros(1,2);
    peakheight = zeros(1,3);
    % find the R wave
    center = round((numel(wave_temp)+1)/2);
    lhalf_qrs = round(center-0.04*fs);
    rhalf_qrs = round(center+0.04*fs);
    [peakheight(2), Rloc] = max(abs(wave_temp(lhalf_qrs:rhalf_qrs)));
    Rloc = lhalf_qrs+Rloc-1;
    
    % we only consider pos R peak 
    % if neg, we reverse reverse the signal
    if wave_temp(Rloc) < 0
        wave_temp = -wave_temp;
    end
    
    % Set the searching window of P wave
    P_start = round(Rloc-0.25*fs);
    P_end = lhalf_qrs;
    
    % detrend signal of interest
    de1 = detrend(wave_temp(P_start:P_end),'linear');
    [peaks,signs] = cjy_peakfind(de1,fs*0.08,0); % peaks is the relative location in de1
    peak_id = find(signs == 1);
    if isempty(peak_id)
        peakheight(1) = 0;
        duration(1) = 0;
    else
        peak_id = peak_id(end); % Set the last peak as the potential P peak in default
        peak_h = peaks(peak_id);
        peak_l1 = peaks(peak_id-1);
        peak_l2 = peaks(peak_id+1);
        duration(1) = (peak_l2-peak_l1)/fs;
        peakheight(1) = de1(peaks(peak_id))-de1(peaks(peak_id-1));
    end

    
    
    % Set the searching window of T wave
    T_start = round(Rloc+0.08*fs);
    T_end = round(Rloc+0.37*fs);
    
    de1 = detrend(wave_temp(T_start:T_end),'linear');
    [peaks,signs] = cjy_peakfind(de1,fs*0.1,0);
    peak_id = find(signs == 1);
    if isempty(peak_id)
        peakheight(3) = 0;
        duration(2) = 0;
    else
        peak_id = peak_id(1); % Set the first peak as Potential T peak in default
        peak_h = peaks(peak_id);
        peak_l1 = peaks(peak_id-1);
        peak_l2 = peaks(peak_id+1);
        duration(2) = (peak_l2-peak_l1)/fs;
        peakheight(3) = de1(peaks(peak_id))-de1(peaks(peak_id-1));
    end
    %peakheight = peakheight/peakheight(2); % normalized with R peak
    peakheight = abs(peakheight);
    


