%% read signal
data_dir = ['D:\ECGdatabase\Challenge2017\training2017' filesep];
addpath(pwd)

% read the signal
fid = fopen([data_dir 'RECORDS'],'r');
if(fid ~= -1)
    RECLIST = textscan(fid,'%s');   
else
    error(['Could not open ' data_dir 'RECORDS for scoring. Exiting...'])
end
fclose(fid);
RECORDS = RECLIST{1};

% read the label
reffile = [data_dir filesep 'REFERENCE.csv'];
fid = fopen(reffile, 'r');
if(fid ~= -1)
    Ref = textscan(fid,'%s %s','Delimiter',',');
else
    error(['Could not open ' reffile ' for scoring. Exiting...'])
end
fclose(fid);
target  = Ref{2};


fs = zeros(1,length(RECORDS));
for i = 1:length(RECORDS)
    fprintf('Processing %d record\n',i);
    [tm,ecg,fs,siginfo]= rdmat([data_dir RECORDS{i}]);
    signal{i} = ecg;
end
%% QRS detector
for i = 1:numel(signal)
    fprintf('processing %d signal\n',i);
    mask{i} = UNSW_ArtifactMask(signal{i},[-5,5],0,fs,0);
    [uqrs{i},uRR{i}] = UNSW_QRSDetector(signal{i},fs,mask{i},0,0);
end
%% cjy_EM
rng(1); % For reproducibility
MU1 = [1 2];
SIGMA1 = [2 0; 0 .5];
MU2 = [-3 -5];
SIGMA2 = [1 0; 0 1];
X = [mvnrnd(MU1,SIGMA1,1000);mvnrnd(MU2,SIGMA2,1000)];

scatter(X(:,1),X(:,2),10,'.')
hold on

options = statset('Display','final');
obj = fitgmdist(X,2,'Options',options);
h = ezcontour(@(x,y)pdf(obj,[x y]),[-8 6],[-8 6]);

% compare the two
tic
theta = cjy_em_gmm(X,2);
toc
obj = fitgmdist(X,2,'Options',options);
toc
%% remove baseline wondering
fs = 300;
[b,a] = butter(5,3/(fs/2));
i = 1;
baseline = myfiltfilt(b,a,signal{i});
subplot(311)
plot(signal{i})
subplot(312)
plot(baseline);
subplot(313)
plot(signal{i}-baseline);

% Extract QT interval 250 - 180

L = 180;
for k = 1:30
    temp_mean = mean(diff(jqrs{k}));
    RR(k) = temp_mean;
    temp_std = std(diff(jqrs{k}));
    RR_std(k) = temp_std;
    id = find(diff(jqrs{k}) < temp_mean + temp_std & diff(jqrs{k}) > temp_mean - temp_std);
    P_list{k} = zeros(numel(id)-1,L);
    for i = 1:numel(id)-1
        if (jqrs{k}(i+1)<L+18)
             i = i +1;
        end
        P_list{k}(i,:) = signal{k}(jqrs{k}(i+1)-L-18+1:jqrs{k}(i+1)-18);
    end
end

%% signal quality with correlation
% find the template
load('train_result');
k = 2;
ecg = signal{k};
Rpeak = jqrs{k};
N = numel(jqrs{k});

L = round(mean(diff(Rpeak)));
point = Rpeak(1:end-1)';
point_list = zeros(N-1,L);
for i = 1:N-1
    point_list(i,:) = point(i):point(i)+L-1;
end
template = mean(ecg(point_list));

for i = 1:32
    plot(ecg(point_list(i,:)));
    hold on
end

cor_list = zeros(1,N);
for i = 1:N-1
    cor_list(i) = max(xcorr(template,ecg(Rpeak(i):Rpeak(i)+L)));
end

%% showTQ
   for TQ_j = 1:5
        for TQ_i = 1:2
            TQ_m = (TQ_i-1)*5+TQ_j+1;
            subplot(5,2,TQ_m-1)
            plot(signal1(RR(TQ_m)+18:RR1(TQ_m+1)-18));
        end
   end
%% 3/8/2017 find the anamoly about the deviation and sample Entropy

histogram(AFEv(target == 'A'))
hold on
histogram(AFEv(target == 'N'))
legend('A','N')
title('AFEv comparison between ''N'' and ''A'' ')

%% Split the data
% This small script it to split the whole trainset
% into two groups. 
% We just use part of it for traing.
train_index = [];
test_index = [];
index_list = ['A','N','O','~'];
target = cell2mat(target);
for i = 1:4
    A_group = find(target == index_list(i));
    L_A = numel(A_group);
    rand_A = randperm(numel(A_group));
    A_1 =  A_group(rand_A(1:round(L_A/2)));
    A_2 =  setdiff(A_group,A_1);
    train_index = [train_index; A_1];
    test_index = [test_index; A_2];
end

% extract the wave
for i = 1:numel(target)
    fprintf("processing %d\n",i);
    m_id = find(uqrs{i}>left & uqrs{i} < length(signal{i}) - right);
    m = numel(m_id);
    wave_mat =  zeros(m,left+right+1);
    for j = 1:m
       wave_mat(j,:) = signal{i}(uqrs{i}(m_id(j))-left:uqrs{i}(m_id(j))+left);
    end
    wave{i} = wave_mat;
end

train_wave = wave(train_index);
train_set = signal(train_index);
train_label = target(train_index);
train_uqrs = uqrs(train_index);
train_RR = uRR(train_index);
train_mask = mask(train_index);

valid_set = signal(test_index);
valid_label = target(test_index);
valid_uqrs = uqrs(test_index);
valid_RR = uRR(test_index);
valid_mask = mask(test_index);
valid_index = test_index;
valid_wave = wave(valid_index);

save('.\dataset\validset.mat','valid_set','valid_label','valid_uqrs','valid_RR','valid_mask','valid_index','valid_wave');
save('.\dataset\trainset.mat','train_set','train_label','train_uqrs','train_RR','train_mask','train_index','train_wave');

%%
load('.\dataset\trainset');
idNoise = find(train_label == '~');
idNormal  = find(train_label == 'N');
idAF  = find(train_label == 'A');
idOther  = find(train_label == 'O');

mask_length = zeros(1,numel(train_label));

for i = 1:numel(mask_length)
    mask_length(i) = length(train_mask{i})/length(train_set{i});
end

boxplot(mask_length,train_label);
title('Mask length as feature');
saveas(gcf,'.\Pic\mask_length.jpg');

% Mask length seems to be a promising feature,
% Let's find if it could be augmented
id_interest = find(mask_length > 0.1 & train_label' ~= '~');
i = 2;
plot(train_set{id_interest(i)});
hold on
plot(train_mask{id_interest(i)},train_set{id_interest(i)}(train_mask{id_interest(i)}));

id_noise = find(train_label == '~');
%% add the jqrs into trainset and validset

load('.\dataset\trainset');
fs = 300;
for i = 1:numel(train_set)
    fprintf('Processing %d\n',i);
    train_jqrs{i} = qrs_detect2(train_set{i}',0.25,0.6,fs);
end
save('.\dataset\trainset.mat','signal','label','uqrs','RR','mask','wave','feature','index');

load('.\dataset\validset');
fs = 300;
for i = 1:numel(valid_set)
    fprintf('Processing %d\n',i);
    valid_jqrs{i} = qrs_detect2(valid_set{i}',0.25,0.6,fs);
end
save('.\dataset\validset.mat','signal','label','uqrs','RR','mask','wave','feature','index');
%% Segment the trainset into small segments for deep learning training
load('.\dataset\trainset.mat');
window_L = 1500; %get approximately 5s downsample by 2
seg_num = 15;
m_train = numel(train_set);
trainsetseg = zeros(seg_num*m_train,window_L);
trainsetseg_label = char(seg_num*m_train,1);
time_list = zeros(1,m_train);
tic
for i = 1:m_train
    fprintf('processing %d signal\n',i); 
    % signal = downsample(train_set{i},2); % if downsample,
    orig = randi(length(train_set{i})-window_L,[seg_num,1]);
    for j = 1:seg_num
        trainsetseg((i-1)*15+j,:) = train_set{i}(orig(j):orig(j)+window_L-1);
        trainsetseg_label((i-1)*15+j,:) = train_label(i);
    end
    time_list(i) = toc;
end
plot(diff(time_list))
trainsetseg = downsample(trainsetseg',2)';
save('.\dataset\trainsegset','trainsetseg','trainsetseg_label');

load('.\dataset\validset.mat');
m_valid = numel(valid_set);
validsetseg = zeros(seg_num*m_valid,window_L);
validsetseg_label = zeros(seg_num*m_valid,1);
for i = 1:m_valid
    fprintf('processing %d signal\n',i);
    % signal = downsample(train_set{i},2); % if downsample,
    orig = randi(length(valid_set{i})-window_L,[seg_num,1]);
    for j = 1:seg_num
        validsetseg((i-1)*15+j,:) = valid_set{i}(orig(j):orig(j)+window_L-1);
        validsetseg_label((i-1)*15+j) = valid_label(i);
    end     
end
validsetseg = downsample(validsetseg',2)';
save('.\dataset\validsegset.mat','validsetseg','validsetseg_label');

%view the activations
plot(activations(convnet,trainsetseg(1,:,1),1))
hold on 
plot(activations(convnet,trainsetseg(1,:,1),2))
%% 03/17 waveform matching
load('.\dataset\trainset.mat');
fs = 300;
left = 0.5*fs;
right = 0.5*fs;
wave = cell(1,numel(target));
% train_cov = cell(1,numel(train_set));
for i = 1:numel(target)
    fprintf("processing %d\n",i);
    m_id = find(uqrs{i}>left & uqrs{i} < length(signal{i}) - right);
    m = numel(m_id);
    wave_mat =  zeros(m,left+right+1);
    for j = 1:m
       wave_mat(j,:) = signal{i}(uqrs{i}(m_id(j))-left:uqrs{i}(m_id(j))+left);
    end
    wave{i} = wave_mat;
end
train_wave = wave(train_index);
valid_wave = wave(valid_index);
wave_feature = zeros(numel(train_set),4);
for i = 1:numel(train_set)
    wave_feature(i,1) = min(train_cor{i});
    wave_feature(i,2) = max(train_cor{i});
    wave_feature(i,3) = mean(train_cor{i});
    wave_feature(i,4) = median(train_cor{i});
end


% to see how the four features are
subplot(221)
boxplot(wave_feature(:,1),train_label);
title('corrcoef min');

subplot(222)
boxplot(wave_feature(:,2),train_label);
title('corrcoef max');

subplot(223)
boxplot(wave_feature(:,3),train_label);
title('corrcoef mean')

subplot(224)
boxplot(wave_feature(:,4),train_label);
title('corrcoef median')
saveas(gcf,'.\Pic\corrcoef_feature.jpg');

% to detect some anomaly
[vsort,psort] = sort(wave_feature(train_label == 'N',3));
histogram(vsort);
title('Histogram for mean corr');
saveas(gcf,'.\Pic\hist.jpg')
id_N = find(train_label == 'N');
m = 1;
for i = 1+(m-1)*4:m*4
subplot(2,2,i-(m-1)*4)
plot(train_set{id_N(psort(i))});
title([ num2str(id_N(psort(i))) '  ' train_label(id_N(psort(i))) '  ' num2str(vsort(i))]);
end
saveas(gcf,['.\Pic\meancorrcoef' num2str(m) '.jpg'])

% find some anomaly, i=1137,3077,869 is quite regular, why get a low score?
i = 1137;
temp = wave{i};
temp_t = train_cor{i};
mean_wave = mean(temp);
plot([mean_wave3; temp([1 2 3 4],:)]');
legend('mean','1','2','3','4');
% Anomaly waveform detection, can we do it.
% I guess the quality of feature will increase if finished it
showQRS(train_set{i},train_uqrs{i},1);
title([num2str(i) '  ' train_label(id_N(psort(i))) ' ' num2str(vsort(2))])
saveas(gcf,'Pic\false peak distorted the xcorcoef')

good_id = find(temp_t>0.5); % not so good !
mean_wave2 = mean(temp(good_id,:));

% try classifying with k-means
k = kmeans(temp,2);
mean_wave3 = temp(k == 2,:);
plot([mean_wave3;mean(mean_wave3)]');

% why I do not use corrcoef
subplot(211)
plot([mean(mean_wave3);temp(5,:)]')
cor_temp = corrcoef(mean(mean_wave3),temp(5,:));
legend('mean wave','5')
title(['corrcoeff is  ' num2str(cor_temp(1,2))]);

subplot(212)
plot([mean(mean_wave3);temp(1,:)]')
cor_temp = corrcoef(mean(mean_wave3),temp(1,:));
legend('mean wave','1')
title(['corrcoeff is  ' num2str(cor_temp(1,2))]);

saveas(gcf,'.\Pic\Notcorrcoef.jpg');

% choose mse to measure the similarity
for i = 1:numel(temp_t)
    newfeature(i) = mean(abs(mean(mean_wave3)-temp(i,:)));
end

subplot(211)
plot([mean(mean_wave3);temp(5,:)]')
legend('mean wave','5')
title(['corrcoeff is  ' num2str(newfeature(5))]);

subplot(212)
plot([mean(mean_wave3);temp(1,:)]')
legend('mean wave','1')
title(['corrcoeff is  ' num2str(newfeature(1))]);

saveas(gcf,'.\Pic\mse.jpg');

% now change our feature, first using k-means
% and mse
train_mse = cell(1,numel(wave));
train_ratio = zeros(1,numel(wave));
for i = 1:numel(wave)
    fprintf("processing %d\n",i);
    k = kmeans(wave{i},2);
    train_ratio(i) = sum(k == mode(k))/numel(k);
    meanwave = mean(wave{i}(k == mode(k),:));
    wavemse = zeros(1,numel(k));
    for j = 1:numel(k)
    wavemse(j) = mean(abs(meanwave-wave{i}(j,:)));
    end

end

wave_feature = zeros(numel(train_set),4);
for i = 1:numel(train_set)
    wave_feature(i,1) = min(train_mse{i});
    wave_feature(i,2) = max(train_mse{i});
    wave_feature(i,3) = mean(train_mse{i});
    wave_feature(i,4) = median(train_mse{i});
    wave_feature(i,5) = sum(train_mse{i}<0.2)/numel(train_mse{i});
end


% to see how the four features are
subplot(221)
boxplot(wave_feature(:,1),train_label);
title('mse min');

subplot(222)
boxplot(wave_feature(:,2),train_label);
title('mse max');

subplot(223)
boxplot(wave_feature(:,3),train_label);
title('mse mean')

subplot(224)
boxplot(wave_feature(:,4),train_label);
title('mse median')
saveas(gcf,'.\Pic\mse_feature.jpg');

% find about Noise
idN = find(train_label == '~');
plot(train_mse{idN(4)});

mkdir('result')
save(".\result\result317_wave_cov_mse.mat",'train_wave','train_cov','train_mse');
%% 03/18 To get more information about train_mse
load('.\result\result317_wave_cov_mse.mat');
load('.\dataset\trainset');
m = numel(wave);
wave_feature = zeros(m,5);

% I want to extract five features
% divide the wave into 2 groups
% ratio, mean,and std for group 1
% ratio, mean,and std for group 2
for i = 1:m
   fprintf("Processing %d\n",i);
   wave = wave{i};
   [k,~,ksum,D] = kmeans(wave,2);
   Dis = min(D,[],2);
   k_max = mode(k);
   wave_feature(i,2) = mean(Dis(k == k_max));
   wave_feature(i,3) = std(Dis(k == k_max));
   wave_feature(i,4) = mean(Dis(k ~= k_max));
   wave_feature(i,5) = std(Dis(k ~=k_max));
end

% not good, I want to normalize the mse
train_norm_mse = cell(1,m);
for i = 1:m
   fprintf("Processing %d\n",i);
   wave = wave{i};
   [k,C,ksum,D] = kmeans(wave,2);
   mse = zeros(1,numel(k));
   k_max = mode(k);
   wave_temp = C(k_max,:);
   amp_temp = mean(abs(wave_temp));
   for j = 1:numel(k)
       mse(j) = mean(abs(wave_temp-wave(j,:)))/(amp_temp*mean(abs(wave(j,:))));
   end
   assert(sum(isinf(mse))==0);
   train_norm_mse{i} = mse;
end

wave_feature = zeros(m,4);
for i = 1:numel(train_set)
    wave_feature(i,2) = min(train_norm_mse{i});
    wave_feature(i,3) = max(train_norm_mse{i});
    wave_feature(i,4) = mean(train_norm_mse{i});
    wave_feature(i,5) = median(train_norm_mse{i});
end

% Hey, I find some problems with the waveform I extract. 
% why the first is always 0.
load('.\result\result318_wave_normmse.mat')
load('.\dataset\trainset.mat')

% combine the futures to build new neural network
%% 03/19 bold attemp, let's put the template in
load('.\result\result318_wave_normmse.mat')
load('.\dataset\trainset.mat')
wave_temp = zeros(numel(wave),301);
for i = 1:m
   fprintf("Processing %d\n",i);
   wave = wave{i};
   [k,C,ksum,D] = kmeans(wave,2);
   mse = zeros(1,numel(k));
   k_max = mode(k);
   wave_temp(i,:) = C(k_max,:);
end
% not so good, acc is acound 57.81%

%% 03/19 Try to extract frequency feature
load('.\dataset\trainset.mat');
idN = find(train_label == 'N');
for i = 1:100
    wtemp = wave_temp(i,:);
    [~,idp] = max(abs(wtemp));
    if wtemp(idp) < 0
        plot(-wtemp);
    else
    plot(wtemp);
    hold on
    end
end 

% frequency analysis - wavelet analysis
wt = modwt(wave_temp(1,:),5);
wtrec = zeros(size(wt));
wtrec(4:5,:) = wt(4:5,:);
y = imodwt(wtrec,'sym4');

m = 2;
i = 5;
subplot(211)
wt1 = modwt(wave_temp(1+m,:),5);
plot(wt1(i,:)');
subplot(212)
wt2 = modwt(wave_temp(1000+m,:),5);
plot(wt2(i,:)');

% compute RWE
m = numel(train_label);
RWE = zeros(numel(train_label),6);
for i = 1:m
    fprintf("processing %d\n",i);
    wavet = wave_temp(i,:);
    wt = modwt(wavet,5);
    wt = sum(wt.^2,2)/sum(sum(wt.^2,2));
    RWE(i,:) = wt';
end
%% 03/19 night modwt comes across some problem in the test platform
% let's transfer it to dwt
load('.\dataset\trainset.mat');
load('.\result\result318_wave_normmse.mat');

RWE = zeros(numel(train_set),6);
for i = 1:numel(train_set)
    fprintf("processing %d\n",i);
    wavet = wave_temp(i,:);
    [C,L] = wavedec(wavet,5,'sym4');
    sumL = cumsum(L);
    R = zeros(1,6);
    R(1) = sum(C(1:sumL(1)).^2);
    for j = 1:5
        R(j+1) = sum(C(sumL(j):sumL(j+1)).^2);
    end
    RWE(i,:) = R/sum(R);
end
%% 3/23 back to project 
% adding Heba and Stephen's features
load('.\dataset\trainset.mat');
m = numel(train_set);
QRS = 135:165; % 0.1s window
feature = zeros(m,8);
for i = 1:m
    fprintf("processing %d\n",i);
    wave_n = size(wave{i},1);
    wave_temp = mean(wave{i});
    norm_wave = norm(wave_temp);
    peak_height = max(abs(wave_temp));
    norm_qrs = norm(wave_temp(QRS));
    feature_mat = zeros(wave_n,5);
    for j = 1:wave_n
        err = norm(wave_temp-wave{i}(j,:));
        feature_mat(j,1) = err/norm_wave;
        feature_mat(j,2) = err/peak_height;
        feature_mat(j,3) = norm(wave_temp(QRS)-wave{i}(j,QRS))/norm_wave;
        cortemp1 = corrcoef(wave_temp,wave{i}(j,:));
        feature_mat(j,4) = cortemp1(1,2);
        cortemp2 = corrcoef(wave_temp(QRS),wave{i}(j,QRS));
        feature_mat(j,5) = cortemp2(1,2);
    end
    feature(i,1:5) = mean(feature_mat);
    feature(i,6) = median(feature_mat(:,4));
    feature(i,7) = median(feature_mat(:,5));
    feature(i,8) = mean(train_RR{i});
end        
%% extract mean hf component
m = numel(train_set);
hfmean = zeros(1,numel(train_set));


% Get HF power (with 50 Hz removed)
% Remove mains frequency
fs = 300;
[b,a] = ellip(5,0.5,80,40/(fs/2),'high');
for i = 1:m
    hfcleandata = myfiltfilt(b,a,train_set{i});
    % smooth the power
    [b]=hamming(round(0.05*fs));b=b/sum(abs(b));[a]=1;
    b=b./sum(abs(b));
    hfpower=real(myfiltfilt(b,a,hfcleandata.^2).^0.5); 
    hfmean(i) = mean(hfpower);
end

boxplot(hfmean,train_label);

% to see why other noise have so much high frequency component
id_N = find(train_label == 'N');
id_n = find(train_label == '~');
id_A = find(train_label == 'A');
id_O = find(train_label == 'O');
[max_nv,max_np] = max(hfmean(id_N));
a = [4629 4631 4633 4634 4635 4637 4644 4648];
%% Correct the wrong classified signal
a = fopen('REFERENCE-v2.csv');
data = textscan(a,'%s%s','Delimiter',',');
train_label = data{2}(train_index);
test_label = data{2}(test_index);
train_label = cell2mat(train_label};
test_label = cell2mat(test_label);
%% 3/31 careful scruitanization on the dataset
vpred(vpred == 1) = 'N';
vpred(vpred == 2) = 'A';
vpred(vpred == 3) = 'O';
vpred(vpred == 4) = '~';
wrongid = find(vpred' ~= vlabel);
%% 3/28 evening To have a detail see of the result
f = fopen('wrongid_valid.txt','w+');
for i = 1:numel(wrongid)
    fprintf(f,"%s  %s  %s\n",num2str(wrongid(i)),vlabel(wrongid(i)),char(vpred(wrongid(i))));
end
fclose(f);
%% 4/1 build the new dataset: wrongdataset
% contain all the records that are wrong classified
f = fopen('wrongid_valid_ew=2.txt');
data =textscan(f,"%d%s%s");
fclose(f)
wrongid = data{1};
label = data{2};
pre = data{3};

load('.\dataset\validset.mat');
signal = valid_set(wrongid);
uqrs = valid_uqrs(wrongid);
wave = valid_wave(wrongid);
label = valid_label(wrongid);
index = valid_index(wrongid);
RR = valid_RR(wrongid);
mask = valid_mask(wrongid);

save('.\dataset\trainset.mat','signal','uqrs','wave','label','index','RR','mask','feature');

load('.\dataset\trainset.mat');
index = train_index;
label = train_label;
mask = train_mask;
RR = train_RR;
signal = train_set;
uqrs = train_uqrs;
wave = train_wave;

% find wrong classifier go with N
f = fopen('wrongid_valid_ew=2.txt');
data =textscan(f,"%d%c%c");
fclose(f);

wrongid = 1:numel(data{1});
label = data{2};
pre = data{3};

idN = find((pre == 'N' | label == 'N')==1);
f = fopen('Nwrong.txt','w+');
for i = 1:numel(idN)
    fprintf(f,"%d %s %s\n",idN(i),label(idN(i)),pre(idN(i)));
end
fclose(f);
%% Model 10 neural network for N and non-N
load('.\dataset\trainset.mat');
feature = zeros(numel(signal),23);
fs = 300;
for i = 1:length(signal)
    fprintf('Processing the %d record\n',i);
    feature(i,:) = cjy_feature(signal{i},uqrs{i},RR{i},fs);
end

% change the label
label_mat = zeros(numel(signal),1);
label_mat(label == 'N') = 1;
label_mat(label == 'A') = 0;
label_mat(label == 'O') = 1;
label_mat(label == '~') = 0;

% adjusting the weight;
%ew = ones(1,numel(signal));
%ew(label == '~')= 1;

% build the feedforward network
net = patternnet([10,10]);
net.PerformFcn = 'crossentropy'; 
net.trainFcn = 'trainlm';
net.performParam.regularization = 0;
net.layers{1}.transferFcn = 'logsig'; % logsig
net = train(net,feature',label_mat',[],[]);

load('.\dataset\validset.mat');
feature = zeros(numel(RR),23);
for i = 1:length(RR)
    fprintf('Valid: Processing the %d record\n',i);
    feature(i,:) = cjy_feature(signal{i},uqrs{i},RR{i},fs);
end
pre = net(feature');
label_mat = (label == 'N');
vpred = pre>0.5;
id_n = find(vpred' ~= label_mat);
%% How about using convolution network for training (not so good)
load('.\dataset\trainset.mat');
feature = zeros(numel(signal),23+301);
fs = 300;
for i = 1:length(signal)
    fprintf('Processing the %d record\n',i);
    feature(i,1:23) = cjy_feature(signal{i},uqrs{i},RR{i},fs);
    feature(i,24:end) = mean(wave{i});
end

% change the label
label_mat = (label == 'N');

% build the cnn
trainsetseg = reshape(feature',[1 324 1 size(feature,1)]);
layers = [imageInputLayer([1 324 1])
convolution2dLayer([1,20],5)
reluLayer
maxPooling2dLayer([1,6],'Stride',[1,6])
fullyConnectedLayer(2)
softmaxLayer
classificationLayer()];
options = trainingOptions('sgdm','MaxEpochs',100, ...
'InitialLearnRate',0.0001);
train_label = categorical(label_mat);
convnet = trainNetwork(trainsetseg,train_label,layers,options);
% bad
%% How abot svm (not so good)
feature = feature(:,1:23);
model = fitcsvm(feature,label_mat);
pre = predict(model,feature);
sum(pre ~= label_mat)/numel(pre);
%% How about random forest trees
B = TreeBagger(50,feature,label_mat,'OOBPrediction','On',...
    'Method','classification');
view(B.Trees{1},'Mode','graph')
vpred = cell2mat(predict(B,feature));
vpred = (vpred == '1');
label_mat = (label==1);
sum(vpred ~= label_mat)
%%
load('.\dataset\trainset.mat');
feature_train = zeros(numel(signal),23);
fs = 300;
for i = 1:length(signal)
    fprintf('Processing the %d record\n',i);
    feature_train(i,:) = cjy_feature(signal{i},uqrs{i},RR{i},fs);
end

% change the label
label_mat = zeros(numel(signal),1);
label_mat(label == 'N') = 1;
label_mat(label == 'A') = 0;
label_mat(label == 'O') = 1;
label_mat(label == '~') = 0;

% build the feedforward network
net1 = patternnet([10,10]);
net1.PerformFcn = 'crossentropy'; 
net1.trainFcn = 'trainlm';
net1.performParam.regularization = 0;
net1.layers{1}.transferFcn = 'logsig'; % logsig
net1 = train(net1,feature_train',label_mat',[],[]);

id_an = find(label == 'A' | label == '~');
feature_an = feature_train(id_an,:);
label_an = label(id_an);
label_an = (label_an == 'A');

net11 = patternnet([10,10]);
net11.PerformFcn = 'crossentropy'; 
net11.trainFcn = 'trainlm';
net11.performParam.regularization = 0;
net11.layers{1}.transferFcn = 'logsig'; % logsig
net11 = train(net11,feature_an',label_an',[],[]);


id_no = find(label == 'N' | label == 'O');
feature_no = feature(id_no,:);
label_no = label(id_no);
label_no = (label_no == 'N');

net12 = patternnet([10,10]);
net12.PerformFcn = 'crossentropy'; 
net12.trainFcn = 'trainlm';
net12.performParam.regularization = 0;
net12.layers{1}.transferFcn = 'logsig'; % logsig
net12 = train(net12,feature_no',label_no',[],[]);

% validate
load('.\dataset\validset.mat');
feature_valid = zeros(numel(RR),23);
for i = 1:length(RR)
    fprintf('Valid: Processing the %d record\n',i);
    feature_valid(i,:) = cjy_feature(signal{i},uqrs{i},RR{i},fs);
end

% no & an
result1 = net1(feature_valid');

feature_no = feature_valid(result1>0.5,:);
label_no = label(result1>0.5);
feature_an = feature_valid(result1<0.5,:);
label_an = label(result1<0.5);

% no
result11 = net11(feature_no');
result11 = (result11>0.5)+1; % 2 for N, 1 for O

% an
result12 = net12(feature_an');
result12 = (result12>0.5)+3; % 4 for A, 3 for ~

valid_label = [label_no;label_an];
ems_vpre = [result11 result12];
confu_mat = zeros(4,4);

for i = 1:numel(valid_label)
    switch valid_label(i)
        case 'N'
            confu_mat(1,ems_vpre(i)) = confu_mat(1,ems_vpre(i))+1;
        case 'O'
            confu_mat(2,ems_vpre(i)) = confu_mat(2,ems_vpre(i))+1;
        case 'A'
            confu_mat(3,ems_vpre(i)) = confu_mat(3,ems_vpre(i))+1;
        case '~'
            confu_mat(4,ems_vpre(i)) = confu_mat(4,ems_vpre(i))+1;
    end
end

F1n=2*confu_mat(1,1)/(sum(confu_mat(1,:))+sum(confu_mat(:,1)));
F1a=2*confu_mat(2,2)/(sum(confu_mat(2,:))+sum(confu_mat(:,2)));
F1o=2*confu_mat(3,3)/(sum(confu_mat(3,:))+sum(confu_mat(:,3)));
F1p=2*confu_mat(4,4)/(sum(confu_mat(4,:))+sum(confu_mat(:,4)));
F1=(F1n+F1a+F1o+F1p)/4;


str = ['F1 measure for Other rhythm:  ' '%1.4f\n'];
fprintf(str,F1n)
str = ['F1 measure for Normal rhythm:  ' '%1.4f\n'];
fprintf(str,F1a)
str = ['F1 measure for noise rhythm:  ' '%1.4f\n'];
fprintf(str,F1o)
str = ['F1 measure for AF recordings:  ' '%1.4f\n'];
fprintf(str,F1p)
str = ['Final F1 measure:  ' '%1.4f\n'];
fprintf(str,F1)
% not so good
%% Learn if I can improve the result by setting the second examination
load('.\dataset\trainset.mat')

feature = zeros(numel(signal),30);
fs = 300;
for i = 1:length(signal)
    fprintf('Processing the %d record\n',i);
    feature(i,:) = cjy_feature(signal{i},uqrs{i},RR{i},fs);
end

prob = net(feature');
[~,vpred] = max(prob);
label(label == 'N') = 1;
label(label == 'A') = 2;
label(label == 'O') = 3;
label(label == '~') = 4;
label = double(label);

wrong = find(label ~= vpred' & (vpred' == 1));
prob_roi = prob(:,wrong);

%% 4/3 add some more features about the amplitude and try adaboost
load('.\dataset\trainset.mat');
feature_train = feature;
label_train = label;
load('.\dataset\validset.mat');
feature = [feature_train;feature];
label = [label_train;label];

for i = 1:length(signal)
    fprintf('Processing the %d record\n',i);
    feature(i,:) = cjy_feature(signal{i},uqrs{i},RR{i},fs);
end

M = 6000; % number of training numbers in Emsemble NN
N = 10; % build ten classifier
Np = 0.4*M;
Ap = 0.18*M;
Op = 0.36*M;
np = 0.06*M;

id_N = randi(sum(label == 'N'),[N,Np]);
id_A = randi(sum(label == 'A'),[N,Ap]);
id_O = randi(sum(label == 'O'),[N,Op]);
id_n = randi(sum(label == '~'),[N,np]);

label_sub = [ones(Np,1);2*ones(Ap,1);3*ones(Op,1); 4*ones(np,1)];
label_mat = zeros(M,4);

label_mat(label_sub == 1, 1) = 1;
label_mat(label_sub == 2, 2) = 1;
label_mat(label_sub == 3, 3) = 1;
label_mat(label_sub == 4, 4) = 1;

ID_N = find(label == 'N');
ID_A = find(label == 'A');
ID_O = find(label == 'O');
ID_n = find(label == '~');

for i = 1:10
    total_ID = [ID_N(id_N(i,:)); ID_A(id_A(i,:)) ;ID_O(id_O(i,:)) ;ID_n(id_n(i,:))];
    feature_sub = feature(total_ID,[1:23]);
    net{i} = patternnet([10,10]);
    net{i}.PerformFcn = 'crossentropy'; 
    net{i}.trainFcn = 'trainlm';
    net{i}.performParam.regularization = 1;
    net{i}.layers{1}.transferFcn = 'logsig'; % logsig
    ew = ones(1,M);
    ew(label == '~')= 2;
    net{i} = train(net{i},feature_sub',label_mat',[],[]);
end

% for i = 1:size(feature_valid,1)
%     fprintf('Processing the %d record\n',i);
%     feature_valid(i,:) = cjy_feature(signal{i},uqrs{i},RR{i},fs);
% end

vpred = zeros(size(feature,1),10);
for i = 1:10
    prob = net{i}(feature');
    [~,vpred(:,i)] = max(prob);
end
ems_vpre = mode(vpred,2);


% prob = net{k}(feature');
% [~,class_valid] = max(prob);
confu_mat = zeros(4,4);
for i = 1:numel(label)
    switch label(i)
        case 'N'
            confu_mat(1,ems_vpre(i)) = confu_mat(1,ems_vpre(i))+1;
        case 'A'
            confu_mat(2,ems_vpre(i)) = confu_mat(2,ems_vpre(i))+1;
        case 'O'
            confu_mat(3,ems_vpre(i)) = confu_mat(3,ems_vpre(i))+1;
        case '~'
            confu_mat(4,ems_vpre(i)) = confu_mat(4,ems_vpre(i))+1;
    end

end
    F1n=2*confu_mat(1,1)/(sum(confu_mat(1,:))+sum(confu_mat(:,1)));
    F1a=2*confu_mat(2,2)/(sum(confu_mat(2,:))+sum(confu_mat(:,2)));
    F1o=2*confu_mat(3,3)/(sum(confu_mat(3,:))+sum(confu_mat(:,3)));
    F1p=2*confu_mat(4,4)/(sum(confu_mat(4,:))+sum(confu_mat(:,4)));
    F1 =(F1n+F1a+F1o+F1p)/4;
    str = ['F1 measure for Normal rhythm:  ' '%1.4f\n'];
    fprintf(str,F1n)
    str = ['F1 measure for AF rhythm:  ' '%1.4f\n'];
    fprintf(str,F1a)
    str = ['F1 measure for other rhythm:  ' '%1.4f\n'];
    fprintf(str,F1o)
    str = ['F1 measure for noise recordings:  ' '%1.4f\n'];
    fprintf(str,F1p)
    str = ['Final F1 measure:  ' '%1.4f\n'];
    fprintf(str,F1)
    
%% find morphological wave
temp = mean(wave{1200});
fs = 300;
% find the R wave
lhalf_qrs = round(151-0.04*fs);
rhalf_qrs = round(151+0.04*fs);
[Rpeakamp, Rloc] = max(abs(temp(lhalf_qrs:rhalf_qrs)));
Rloc = lhalf_qrs+Rloc-1;
P_start = round(Rloc-0.25*fs);
T_start = round(Rloc+0.08*fs);
T_end = round(Rloc+0.37*fs);

% local detrend (P wave)
plot(temp(P_start:lhalf_qrs));
hold on
de1 = detrend(temp(P_start:lhalf_qrs),'linear');
plot(de1);
plot(de1-temp(P_start:lhalf_qrs));
legend('ori','detrend');

[peaks signs] = cjy_peakfind(de1,3,1);
peak_id = find(signs == 1);
peak_id = peak_id(end);
peak_h = peaks(peak_id);
if isempty(peak_id)
    height = 0;
    duration = 0;
else
    peak_l1 = peaks(peak_id-1);
    peak_l2 = peaks(peak_id+1);
    duration = (peak_l2-peak_l1)/fs;
    height = peaks(peak_id)-peaks(peak_id-1);
end


    
% if two peaks are so closed, we delete them

% using minimal as the threshold

duration = diff(t_point)/300; 
% local detrend (T wave)
figure
plot(temp(T_start:T_end));
hold on
de2 = detrend(temp(T_start:T_end),'linear');
plot(de2);
plot(de2-temp(T_start:T_end));
legend('ori','detrend');


% find the PQ point (start point of QRS)
if temp(Rloc) < 0
    temp = -temp;
end

k = (temp(Rloc)-temp(P_start))/(Rloc-P_start);
y_comp = ([P_start : lhalf_qrs]-P_start)*k+temp(P_start);
[iso,iloc] = max(y_comp-temp(P_start:lhalf_qrs));
iloc = iloc+P_start-1;
    
plot(temp)
hold on
plot(iloc,temp(iloc),'*');
line([P_start,Rloc],[temp(P_start),temp(Rloc)]);
%% 4/6 Find the wrong classified record
load('.\dataset\trainset.mat');
vpred = zeros(size(feature,1),10);
for i = 1:10
    prob = net{i}(feature');
    [~,vpred(:,i)] = max(prob);
end
ems_vpre = mode(vpred,2);
idwrong = cell(4,4);
confu_mat = zeros(4,4);
for i = 1:numel(label)
    switch label(i)
        case 'N'
            confu_mat(1,ems_vpre(i)) = confu_mat(1,ems_vpre(i))+1;
            idwrong{1,ems_vpre(i)} = [idwrong{1,ems_vpre(i)} i];
        case 'A'
            confu_mat(2,ems_vpre(i)) = confu_mat(2,ems_vpre(i))+1;
            idwrong{2,ems_vpre(i)} = [idwrong{2,ems_vpre(i)} i];            
        case 'O'
            confu_mat(3,ems_vpre(i)) = confu_mat(3,ems_vpre(i))+1;
            idwrong{3,ems_vpre(i)} = [idwrong{3,ems_vpre(i)} i];            
        case '~'
            confu_mat(4,ems_vpre(i)) = confu_mat(4,ems_vpre(i))+1;
            idwrong{4,ems_vpre(i)} = [idwrong{4,ems_vpre(i)} i];            
    end

end
%% 4/7 for the midterm inspection
load('.\dataset\trainset.mat')
%% the F1 score?
a = 100;
b = 0:100;
f1 = 2*a*b./(a+b);
ave = (a+b)/2;
plot(b,[f1;ave;b]);
xlabel('Spe')
ylabel('F1score or mean')
legend('F1score','mean');
title('Comparison between F1score and mean');

i = 3002;
rng(1)
[c,k] = kmeans(wave{i},2);
wave_major = wave{i}(c==1,:);
wave_temp = mean(wave{i});
plot(wave_major','g');
hold on
plot(wave_temp,'color','k','linewidth',2);
grid minor

%% create picture for each class
load('.\dataset\trainset');

% Atrial fibrillation
group = ['A' 'N' '~' 'O'];
N = 20;
id = zeros(4,N);

for j = 1:4
    temp = find(label == group(j));
    id(j,:) = temp(1:20);
end

for j = 1:4
    for i = 1:N
        L = length(signal{id(j,i)});
        showQRS(signal{id(j,i)},uqrs{id(j,i)},300,1:4500)
        s = [group(j) num2str(id(j,i))];
        title(s)
        saveas(gcf,['.\Pic\' s '.jpg']);
        close all
    end
end

subtightplot(2,1,1)
showQRS(signal{1},uqrs{1},300,1:4500)
subtightplot(2,1,2)
showQRS(signal{2},uqrs{2},300,1:4500);

figure
subplot(2,1,1)
showQRS(signal{1},uqrs{1},300,1:4500)
subplot(2,1,2)
showQRS(signal{2},uqrs{2},300,1:4500);
%% build new GUI
id_N = find(label == 'N');
id_O = find(label == 'O');
id_A = find(label == 'A');
id_n = find(label == '~');

N = signal(id_N(1:20));
O = signal(id_O(1:20));
A = signal(id_A(1:20));
n = signal(id_n(1:20));

all{1} = N;
all{2} = O;
all{3} = A;
all{4} = n;

Nnum = zeros(1,20);
Onum = zeros(1,20);
Anum = zeros(1,20);
nnum = zeros(1,20);
slength = zeros(4,20);
for i = 1:4
    for j = 1:20
        slength(i,j) = length(all{i}{j});
    end
end

slength = zeros(1,4265);

for i = 1:4265
    slength(i) = length(signal{i});
end

l90 = find(slength == 9000);
label90 = label(l90);
N = signal(id_N(1:20));
O = signal(id_O(1:20));
A = signal(id_A(1:20));
n = signal(id_n(1:20));
id_N = find(label90 == 'N');
id_O = find(label90 == 'O');
id_A = find(label90 == 'A');
id_n = find(label90 == '~');

ID_N = l90(id_N);
ID_O = l90(id_O);
ID_A = l90(id_A);
ID_n = l90(id_n);

N = signal(ID_N(1:20));
O = signal(ID_O(1:20));
A = signal(ID_A(1:20));
n = signal(ID_n(1:20));

all{1} = N;
all{2} = O;
all{3} = A;
all{4} = n;

Nnum = zeros(1,20);
Onum = zeros(1,20);
Anum = zeros(1,20);
nnum = zeros(1,20);
slength = zeros(4,20);
for i = 1:4
    for j = 1:20
        slength(i,j) = length(all{i}{j});
    end
end

learn_id = [ID_N(1:20) ID_O(1:20) ID_A(1:20) ID_n(1:20)];
learn_label = label(learn_id);
learn_signal = signal(learn_id);
learn_uqrs = uqrs(learn_id);
learn_RR = RR(learn_id);
learn_wave = wave(learn_id);
learn_feature = feature(learn_id);

load('Model10 ensembleNN+30feature')
vpred = zeros(size(feature,1),10);
for i = 1:10
    prob = net{i}(feature');
    [~,vpred(:,i)] = max(prob);
end

learn_pred = vpred(learn_id);
learn_pred(learn_pred == 1) = 'N';
learn_pred(learn_pred == 2) = 'A';
learn_pred(learn_pred == 3) = 'O';
learn_pred(learn_pred == 4) = '~';

save('.\dataset\learn.mat','learn_id','learn_label','learn_signal','learn_uqrs','learn_RR','learn_wave','learn_pred','learn_feature','learn_mask')
%% store the mask for the learn set
learn_mask = cell(80,1);
for i = 1:80
    learn_mask{i} = cell(1,4);
    fprintf("%d\n",i);
    [learn_mask{i}{4},learn_mask{i}{1},learn_mask{i}{2},learn_mask{i}{3}] = UNSW_ArtifactMask(learn_signal{i},[-5,5],0,300,0);
end
%% automatically draw
app = learnchallenge;
for i = 1:5
    app.IDSpinner.Value = i;
    app.showButton.ButtonPushedFcn(app,[])
    name = strcat('.\learning\',num2str(i),'.png');
    screencapture(app.UIFigure,[],name);  % capture the entire figure into file
end
%% read new version of label? -- no
f = fopen('REFERENCE-v2.csv','r');
data1 = textscan(f,"%s%s","Delimiter",',');
fclose(f);
label1 = cell2mat(data1{2});
f = fopen('REFERENCE-v22.csv','r');
data2 = textscan(f,"%s%s","Delimiter",',');
fclose(f);
label2 = cell2mat(data2{2});
sum(label1 ~= label2)
%% Make some changes on the PQRST feature
% now generate new learn_feature
for i = 1:80
    fprintf("%d\n",i);
    learn_feature(i,:) = cjy_feature(learn_signal{i},learn_uqrs{i},learn_RR{i},300);
end


%% build model using train+valid+test
load('.\dataset\train_py.mat')
load('.\dataset\test_py.mat')
load('.\dataset\valid_py.mat')

M = 5970; % number of training numbers in Emsemble NN
N = 10; % build ten classifier
Np = round(0.4*M);
Ap = round(0.20*M);
Op = round(0.35*M);
np = round(0.05*M);

[~,train_list] = max(train_label,[],2);
[~,valid_list] = max(valid_label,[],2);
[~,test_list] = max(test_label,[],2);

id_N = randi(sum(train_list == 2),[N,Np]);
id_A = randi(sum(train_list == 1),[N,Ap]);
id_O = randi(sum(train_list == 3),[N,Op]);
id_n = randi(sum(train_list == 4),[N,np]);



ID_N = find(train_list == 2);
ID_A = find(train_list == 1);
ID_O = find(train_list == 3);
ID_n = find(train_list == 4);
hidden1 = 10+randi(10,[1,10]);
hidden2 = 10+randi(10,[1,10]);
for i = 1:10
    total_ID = [ID_N(id_N(i,:)); ID_A(id_A(i,:)) ;ID_O(id_O(i,:)) ;ID_n(id_n(i,:))];
    label_mat_sub = train_label(total_ID,:);
    feature_sub = train_feature(total_ID,:);
    net{i} = patternnet([hidden1(i),hidden2(i)]);
    net{i}.PerformFcn = 'crossentropy'; 
    net{i}.trainFcn = 'trainlm';
    net{i}.performParam.regularization = 0;
    net{i}.layers{1}.transferFcn = 'logsig'; % logsig
%     ew = ones(1,M);
%     ew(label == '~')= 2;
    net{i} = train(net{i},feature_sub',label_mat_sub',[],[]);
end

% test on the validation set
vpred = zeros(size(valid_feature,1),10);
for i = 1:10
    prob = net{i}(valid_feature');
    [~,vpred(:,i)] = max(prob);
end
ems_vpre = mode(vpred,2);

vF1_list = zeros(12,5);
for i = 1:10
    vF1_list(i,:) = F1score(valid_list,vpred(:,i));
end
vF1_list(11,:) = F1score(valid_list,ems_vpre); 

% test on the test set
tpred = zeros(size(test_feature,1),10);
for i = 1:10
    prob = net{i}(test_feature');
    [~,tpred(:,i)] = max(prob);
end
ems_tpre = mode(tpred,2);

tF1_list = zeros(12,5);
for i = 1:10
    tF1_list(i,:) = F1score(test_list,tpred(:,i));
end
tF1_list(11,:) = F1score(test_list,ems_tpre); 

% with just one neural network
net{11} = patternnet([10,10]);
net{11}.PerformFcn = 'crossentropy'; 
net{11}.trainFcn = 'trainlm';
net{11}.performParam.regularization = 0;
net{11}.layers{1}.transferFcn = 'logsig'; % logsig
net{11} = train(net{11},train_feature',train_label',[],[]);

prob = net{11}(test_feature');
[~,tpred_11] = max(prob);

prob = net{11}(valid_feature');
[~,vpred_11] = max(prob);
tF1_list(12,:) = F1score(test_list,tpred_11);
vF1_list(12,:) = F1score(valid_list,vpred_11);
save('bagging_result_5970','vF1_list','tF1_list','net')
%% How about 10 neural network using the same 
load('.\dataset\train_py.mat')
load('.\dataset\test_py.mat')
load('.\dataset\valid_py.mat')

hidden1 = 10+randi(10,[1,10]);
hidden2 = 10+randi(10,[1,10]);

for i = 1:10
    net{i} = patternnet([hidden1(i),hidden2(i)]);
    net{i}.PerformFcn = 'crossentropy'; 
    net{i}.trainFcn = 'trainlm';
    net{i}.performParam.regularization = 0;
    net{i}.layers{1}.transferFcn = 'logsig'; % logsig
%     ew = ones(1,M);
%     ew(label == '~')= 2;
    net{i} = train(net{i},train_feature',train_label',[],[]);
end

% test on the validation set
vpred = zeros(size(valid_feature,1),10);
for i = 1:10
    prob = net{i}(valid_feature');
    [~,vpred(:,i)] = max(prob);
end
ems_vpre = mode(vpred,2);

vF1_list = zeros(11,5);
for i = 1:10
    vF1_list(i,:) = F1score(valid_list,vpred(:,i));
end
vF1_list(11,:) = F1score(valid_list,ems_vpre); 

% test on the test set
tpred = zeros(size(test_feature,1),10);
for i = 1:10
    prob = net{i}(test_feature');
    [~,tpred(:,i)] = max(prob);
end
ems_tpre = mode(tpred,2);

tF1_list = zeros(11,5);
for i = 1:10
    tF1_list(i,:) = F1score(test_list,tpred(:,i));
end
tF1_list(11,:) = F1score(test_list,ems_tpre); 

%% How about total changing the structure of the nn

n_layer = 1+randi(2,[1,10]);
hidden_list = cell(1,10);
for i = 1:10
    hidden_list{i} = 10+randi(10,[1,n_layer(i)]);
end

for i = 1:10
    net{i} = patternnet(hidden_list{i});
    net{i}.PerformFcn = 'crossentropy'; 
    net{i}.trainFcn = 'trainlm';
    net{i}.performParam.regularization = 0;
    net{i}.layers{1}.transferFcn = 'logsig'; % logsig
%     ew = ones(1,M);
%     ew(label == '~')= 2;
    net{i} = train(net{i},train_feature',train_label',[],[]);
end

% test on the validation set
vpred = zeros(size(valid_feature,1),10);
for i = 1:10
    prob = net{i}(valid_feature');
    [~,vpred(:,i)] = max(prob);
end
ems_vpre = mode(vpred,2);

vF1_list = zeros(11,5);
for i = 1:10
    vF1_list(i,:) = F1score(valid_list,vpred(:,i));
end
vF1_list(11,:) = F1score(valid_list,ems_vpre); 

% test on the test set
tpred = zeros(size(test_feature,1),10);
for i = 1:10
    prob = net{i}(test_feature');
    [~,tpred(:,i)] = max(prob);
end
ems_tpre = mode(tpred,2);

tF1_list = zeros(11,5);
for i = 1:10
    tF1_list(i,:) = F1score(test_list,tpred(:,i));
end
tF1_list(11,:) = F1score(test_list,ems_tpre); 

%% test on the validationset to find the best combination
[a,b] = ecg_genetic(vpred,valid_list);
tpred1 = mode(tpred(:,logical(a(1,:))),2);
F1score(test_list,tpred1,1)

%% to load the basic model result in excel
load('hypersearch_result.mat');
%% to train 10 basic model
load('.\dataset\train_py.mat')
load('.\dataset\test_py.mat')
load('.\dataset\valid_py.mat')
net = cell(1,10);
[~,valid_list] = max(valid_label,[],2); 
for i = 1:10
    net{i} = patternnet([10 10]);
    net{i}.PerformFcn = 'crossentropy'; 
    net{i}.trainFcn = 'trainlm';
    net{i}.performParam.regularization = 0;
    net{i}.layers{1}.transferFcn = 'logsig'; % logsig
    net{i}.trainParam.showWindow = 1;
    net{i} = train(net{i},train_feature',train_label',[],[]);
    
end

F1list = zeros(1,10);
for i = 1:10
    prob = net{i}(valid_feature');
    [~,pred] = max(prob);
    F1list(i) = F1score(valid_list,pred);
end
save('basic_model','net','F1list');

%%
