function [best_w,best_F1] = ecg_genetic(pred,label)
% Finds the optimal threshold with gene algorithm
% Input
% - net  1*M cell, the input M component networks
% - pred M*N mat,  the prediction of N neural network, N the number of ins
% - pref 1*N mat,  the label

% Output
% - best_w, 1*M, the optimal weight for M networks
% - best_F1, the optimal F1score

%% param
N = size(pred,2); % the number of network
pop_num = 100;
cross_prob = 0.5;
mut_prob = 0.1;
n_generation = 100;
%% initialize the population
pop = randi(2,[pop_num,10])-1;
ada_pop = zeros(1,pop_num);
for i = 1:pop_num
    ada_pop(i) = adafcn(pop(i,:),pred,label);
end

%%
for i = 1:n_generation 
    % sort the pop
    [ada_pop,ada_ind] = sort(ada_pop,'descend');
    pop = pop(ada_ind,:);
    
    % crossover
    cross_list = rand(1,floor(pop_num/4)) > 1-cross_prob;
    cross_pari = find(cross_list == 1);
    cross_pop = zeros(2*numel(cross_pari),N);
    if ~isempty(cross_list)
        for j = 1:numel(cross_pari)
            cross_pop(2*j-1:2*j,:) = crossover(pop(2*cross_pari(j)-1:2*cross_pari(j),:));
        end
    end
    
    % mutate
    mut_list = find(rand(1,pop_num) > 1-mut_prob == 1);
    mut_pop = zeros(numel(mut_list),N);
    if ~isempty(mut_list)
        for j = 1:numel(mut_list)
            mut_pop(j,:) = mutate(pop(mut_list(j),:));
        end
    end
    % the number of supplemenrary individual need to be refilled 
    % in order to keep the total number of 100
    sup_num = pop_num-numel(cross_pari)*2-numel(mut_list);
        
    % keep the good gene
    if sup_num > 0
        next_pop = pop(1:sup_num,:);
        pop = [next_pop;cross_pop;mut_pop];
    else % too high crossover_prob and mut_prob
        cross_prob = cross_prob*0.9;
        mut_prob = mut_prob*0.95;
    end
    
    [ada_pop,ada_ind] = sort(ada_pop,'descend');
    pop = pop(ada_ind,:);

end    

    best_w = pop(1:30,:);
    best_F1 = ada_pop(1:30);
%% adaptive function
function ada = adafcn(ind,pred,label)
    sel_pred = pred(:,logical(ind));
    sel_result = mode(sel_pred,2);
    %ada = mean(sel_result == label);
    ada = F1score(sel_result,label);
    
function child = crossover(parent)
    child = parent;
    L = size(parent,2);
    cross_ind = randi(L,1);
    child(1,cross_ind:end) = parent(2,cross_ind:end);
    child(2,cross_ind:end) = parent(1,cross_ind:end);

function new_ind = mutate(ind)
    L = length(ind);
    mut_ind = randi(L,1);
    new_ind = ind;
    new_ind(mut_ind) = mod(new_ind(mut_ind)+1,2);
