function showQRS_modi(axes,ECG,ann,fs,mask,range)
% this function is specifically designed to show 
% the QRS peak in its corresponding ECG signal.

% Input
% axes                     the figure axes
% ECG           double 1*N Orignal signal
% QRS           int    1*M R peak locations
% range         int    1*L Range for display. range(1) as the start
%                          and range(end) as the end
% fs            int    1*1 Sample frequency for transferring location to time
% mask          cell   1*4 Noise mask returned by UNSW QRS Detector.
%                          cell{1,2,3,4} is high_frequency mask (blue),
%                          low_power mask(green),
%                          base_shift mask (red), rail voltage mask(yellow)
if nargin < 6
    range = [];
end
%% The special case
% 1. the input range is not specified
% 2. the input range exceeds the total length of signal
% In both of the above cases, the total signal will be displayed
if ~isempty(range)
    if length(ECG) > range(end)-range(1)
        pstart = range(1);
        pend = range(end);
    else
        pstart = 1;
        pend = length(ECG);
    end
else
    pstart = 1;
    pend = length(ECG);
    warning('the input range is empty or exceeds the total signal length, display the whole signal instead');
end

%% plot

% four mask
hf_mask = mask{1};
lp_mask = mask{2};
bs_mask = mask{3};
rc_mask = mask{4};

% limit to range specify by the input range

ann = ann(ann > pstart & ann < pend);
hf_mask = hf_mask(hf_mask > pstart & hf_mask < pend);
lp_mask = lp_mask(lp_mask > pstart & lp_mask < pend);
bs_mask = bs_mask(bs_mask > pstart & bs_mask < pend);
rc_mask = rc_mask(rc_mask > pstart & rc_mask < pend);

% plot the original signal
% plot the R peak locations
% plot the mask (hf and lp is above-zero line; bs and rc are below zero line)

% get the amplitude of signal
amp_max = max(ECG(pstart:pend)); % hf 1/2 amp_max, 3/4 amp_max
amp_min = min(ECG(pstart:pend)); % bs -1/2 amp_min, -3/4 amp_min

plot(axes,(pstart:pend)/fs,ECG(pstart:pend));
hold(axes,'on')
plot(axes,ann/fs,ECG(ann),'o');
seg_plot(axes,hf_mask,amp_max*ones(1,length(hf_mask)),fs,'g');
seg_plot(axes,lp_mask,1.5*amp_max*ones(1,length(lp_mask)),fs,'b');
seg_plot(axes,bs_mask,1.5*amp_min*ones(1,length(bs_mask)),fs,'r');
seg_plot(axes,rc_mask,amp_min*ones(1,length(rc_mask)),fs,'y');
grid(axes,'on');
xlabel(axes,'time(seconds)')
ylabel(axes,'ECG signal')
%legend([h1,h3,h4,h5,h6],'signal','hf','lp','bs','rc')
end

 function seg_plot(axes,x,y,fs,color)
     % This function is a derivative of plot
     % Specifically designed to plot discontinity line
     % for the UNSW Mask plot
     
     % To find the break point
     % for each discontinunity point we find, there are two points.
     % and the number of segments is the number of discontinunity point + 1
     id = find(diff(x) > 1);
     point_end = zeros(1,2*numel(id));
     for i = 1:numel(id)
        point_end(2*i-1) = id(i);
        point_end(2*i) = id(i)+1;
     end
     point_end = [1 point_end length(x)];
     for i = 1:numel(id)+1
         plot(axes,x(point_end(2*i-1):point_end(2*i))/fs,y(point_end(2*i-1):point_end(2*i)),...
             'color',color,'LineWidth',2)
     end
 end